/* 推導盛中唐擬音
 *
 * 中古漢語音變匯總：https://zhuanlan.zhihu.com/p/403479390
 * 本擬音是根據南北朝以來的音變，從《切韻》音系推出的。盛中唐期間的一些音變，以及擬音和記音方面的多種可能，均以選項形式列出
 *
 * 擬音依據包括：
 * - 韻圖
 * - 玄應、慧琳音義和各家反切
 * - 詩歌碑誌銘文用韻
 * - 日、韓、越漢字音
 * - 梵語、藏語、突厥語、粟特語、回鶻語譯音
 * - 關於語音不正的記載和笑話
 *
 * @author unt
 */

const is = (x) => 音韻地位.屬於(x);

if (!音韻地位) return [
  ['$legacy', true],
  ['預置方案（會覆蓋後面的選項）：', [1,
    '無',
    '【老派/約700年】區分常船、BC韻、咍泰、清青，莊組tʂ，章組歸韻圖四等，未濁上歸去',
    '【中派/約750年】不分常船、BC韻、咍泰、清青，莊組tʂ，章組歸韻圖四等',
    '【新派/約800年】不分常船、BC韻、咍泰、清青，莊章合爲照組，濁上歸去，江韻歸宕',
  ]],
  ['非奉：', [1, 'pɸ bβ', 'pf bv', 'ɸ β', 'f v']],
  ['微母：', [2, 'mβ', 'mʋ', 'β̃', 'ɱ', 'ʋ̃']], // 微母脫鼻比非母脫塞晚很多
  ['莊組：', [1, 'tʂ', 'tɕ']], // 到了韻圖音系（790 年左右），莊二、章四轉爲照二、照三，則應寫成 tɕ 組
  ['常船母：', [2, '常dʑ 船ʑ', '平dʑ 仄ʑ', '全dʑ', '全ʑ']],
  ['二等介音：', [1, 'ʕ', 'ʁ', 'ɣ', 'ɻ']],
  ['章組及日母介音：', [2, '韻圖三等', '韻圖四等']],
  ['幫組一等介音：', [1, '無', 'w']],
  ['宕江韻基合流', false],
  ['佳韻分入皆麻', true],
  ['部分唇音流攝歸遇攝', true],
  ['BC韻合流', true],
  ['咍泰合併', true],
  ['清青合併', true],
  ['精莊組止攝開口：', [2, 'i', 'ɹ̩']], // 另有章組止攝開口舌冠化也發生于這個時期，但不被認爲是當時通語的音變，故不包含
  ['魚虞模：', [2, 'j̈ɯ ɥ̈u u', 'j̈ɤ ɥ̈u u', 'j̈ɤ ɥ̈o o']], // 模是介於 u、o 之間的音，默認寫作 u；但魚就用 ɤ 了
  ['侯東韻：', [2, 'əw(ŋ)', 'ɵw(ŋ)', 'ow(ŋ)']],
  ['入聲韻尾：', [2, '-p -t -k', '-β -ɾ -ɣ']], // 在盛中唐已開始弱化
  ['聲調：', [1, '五度符號', '附加符號', '調類數字']],
  ['全濁上歸去', true],
  ['次濁入歸陰入', false],
];

for (var key in 選項) {
  if (key.includes('：')) {
    選項[key.slice(0, -1)] = 選項[key]; // 去除冒號，方便下面代碼中引用
  }
  let 預置方案 = 選項['預置方案（會覆蓋後面的選項）'];
  if (預置方案 !== '無') {
    if (預置方案[1] === '老') {
      選項.常船母 = '常dʑ 船ʑ';
      選項.全濁上歸去 = false;
    } else if (選項.常船母 === '常dʑ 船ʑ') {
      選項.常船母 = '平dʑ 仄ʑ';
    }
    選項.BC韻合流 = 預置方案[1] === '老' ? false : true;
    選項.咍泰合併 = 預置方案[1] === '老' ? false : true;
    選項.清青合併 = 預置方案[1] === '老' ? false : true;
    選項.莊組 = 預置方案[1] === '新' ? 'tɕ' : 'tʂ';
    選項.章組及日母介音 = 預置方案[1] === '新' ? '韻圖三等' : '韻圖四等';
    if (預置方案[1] === '新') {
      選項.全濁上歸去 = true;
      選項.宕江韻基合流 = true;
    }
  }
}

function get韻圖等() {
  // 切韻一二四等到韻圖不變
  if (音韻地位.等 !== '三') return 音韻地位.等;
  // 切韻三等，聲母是銳音的情況
  if (is('莊組')) return '二';
  if (is('知組 或 來母')) return '三';
  if (is('章組 或 日母')) return 選項.章組及日母介音 === '韻圖三等' ? '三' : '四';
  if (is('精組 或 以母')) return '四';
  // 切韻三等，聲母是鈍音的情況
  if (is('重紐A類 或 麻清韻 或 見影組 幽韻')) return '四';
  if (is('幫組 東鍾微虞廢文元陽尤凡韻')) return '輕'; // 輕唇算作一個獨立的等
  return '三';
}

let 韻圖等 = get韻圖等();

function 聲母預處理() {
  // 輕唇音
  let 非奉 = 選項.非奉.split(' ');
  if (is('幫母 東鍾微虞廢文元陽尤凡韻 三等')) return 非奉[0];
  if (is('滂母 東鍾微虞廢文元陽尤凡韻 三等')) return 非奉[0] + 'ʰ';
  if (is('並母 東鍾微虞廢文元陽尤凡韻 三等')) return 非奉[1];
  if (is('明母 鍾微虞廢文元陽凡韻')) return 選項.微母;
  
  // 輕唇音以外的聲母
  return {
    // 儘管全濁聲母在盛中唐在音值上已經弛化乃至清化，但是仍可使用濁音符號
    // 鼻音塞化也不需要寫出來，而且它只是長安音的特色
    幫: 'p',  滂: 'pʰ',  並: 'b',  明: 'm',
    端: 't',  透: 'tʰ',  定: 'd',  泥: 'n', 來: 'l',
    知: 'tɹ', 徹: 'tɹʰ', 澄: 'dɹ', 孃: 'n',
    精: 'ts', 清: 'tsʰ', 從: 'dz', 心: 's', 邪: 'z',
    莊: 'tʂ', 初: 'tʂʰ', 崇: 'dʐ', 生: 'ʂ', 俟: 'dʐ',
    章: 'tɕ', 昌: 'tɕʰ', 常: 'dʑ', 書: 'ɕ', 船: 'ʑ',  日: 'ɲ',
    見: 'k',  溪: 'kʰ',  羣: 'ɡ',  疑: 'ŋ',
    影: 'ʔ',  曉: 'x',   匣: 'ɣ',  云: '',  以: '',
  }[音韻地位.母];
}

function get聲母() {
  let 聲母 = 聲母預處理();
  if (聲母 === 'dʑ' || 聲母 === 'ʑ') {
    if (選項.常船母 === '平dʑ 仄ʑ') {
      聲母 = is('平聲') ? 'dʑ' : 'ʑ';
    } else if (選項.常船母 === '全dʑ') {
      聲母 = 'dʑ';
    } else if (選項.常船母 === '全ʑ') {
      聲母 = 'ʑ';
    }
  }
  if (選項.莊組 === 'tɕ') {
    聲母 = 聲母.replace('ʂ', 'ɕ').replace('ʐ', 'ʑ');
  }
  return 聲母;
}

function 韻母預處理() {
  // 分開合的韻，用豎線分隔開口韻母和合口韻母，之後篩選
  if (is('遇攝') ||
      (選項.部分唇音流攝歸遇攝 && is('流攝') && '浮戊母罘浮蜉矛茂覆懋拇某負阜+謀部畝畮婦不否桴富牟缶'.includes(字頭))) {
      // 加號前是慧琳反切體現的流攝歸遇攝字，加號後是唐代用韻體現的
    if (韻圖等 === '輕') return 'o';
    if (韻圖等 === '一') return 'o';
    if (韻圖等 === '二') return 'ʕɤ|ʕo';
    if (韻圖等 === '三') return 'j̈ɤ|ɥ̈o';
    if (韻圖等 === '四') return 'jɤ|ɥo';
  }
  if (is('果假攝') ||
      (選項.佳韻分入皆麻 && is('佳韻') && '崖咼扠涯搋派差絓畫罣罷+佳鼃娃解釵卦柴'.includes(字頭))) {
      // 加號前是慧琳反切體現的佳韻歸麻韻字，加號後是唐代用韻體現的
    if (韻圖等 === '一') return 'ɑ|wɒ';
    if (韻圖等 === '二') return 'ʕa|wʕa';
    if (韻圖等 === '三') return 'j̈ɑ|ɥ̈ɑ';
    if (韻圖等 === '四') return 'ja|ɥa';
  }
  if (is('止攝')) {
    if (韻圖等 === '二') return 'ʕi|wʕi';
    if (韻圖等 === '三') return !選項.BC韻合流 && is('微韻') ? 'ɨj|ʉj' : 'j̈i|ɥ̈i';
    if (韻圖等 === '輕') return !選項.BC韻合流 ? 'ɨj' : 'j̈i';
    if (韻圖等 === '四') return 'i|ɥi';
  }
  if (is('蟹攝')) {
    if (韻圖等 === '一' && 選項.咍泰合併) return 'ɑj|wɔj';
    if (韻圖等 === '一') return is('泰韻') ? 'ɑj|wɒj' : 'əj|wɔj';
    if (韻圖等 === '二') return 'ʕaj|wʕaj';
    if (韻圖等 === '三') return 選項.BC韻合流 || is('廢韻') ? 'j̈əj|ɥ̈əj' : 'j̈ej|ɥ̈ej';
    if (韻圖等 === '四') return 'jej|ɥej';
    if (韻圖等 === '輕') return 選項.BC韻合流 ? 'jej|ɥej' : 'jəj|ɥəj';
  }
  if (is('流攝')) {
    if (韻圖等 === '輕') return 'ɵw';
    if (韻圖等 === '一') return 'ɵw';
    if (韻圖等 === '二') return 'ʕɵw';
    if (韻圖等 === '三') return 'ɨw';
    if (韻圖等 === '四') return 'iw';
  }
  if (is('效攝')) {
    if (韻圖等 === '一') return 'ʌw';
    if (韻圖等 === '二') return 'ʕaw';
    if (韻圖等 === '三') return 選項.BC韻合流 ? 'j̈ɐw' : 'j̈ɛw';
    if (韻圖等 === '四') return 'jɛw';
  }
  if (is('曾攝')) {
    if (韻圖等 === '一') return 'əŋ|uŋ';
    if (韻圖等 === '二') return 'ʕəŋ|ʕuŋ';
    if (韻圖等 === '三') return 'ɨŋ|ɥ̈ɨŋ';
    if (韻圖等 === '四') return 'jɨŋ|ɥɨŋ';
  }
  if (is('宕攝')) {
    if (韻圖等 === '輕') return 'ɑŋ';
    if (韻圖等 === '一') return 'ɑŋ|wɑŋ';
    if (韻圖等 === '二') return 'ʕɑŋ|wʕɑŋ';
    if (韻圖等 === '三') return 'j̈ɑŋ|ɥ̈ɑŋ';
    if (韻圖等 === '四') return 'jɑŋ|ɥɑŋ';
  }
  if (is('梗攝')) {
    if (韻圖等 === '二') return 'ʕɛjŋ|wʕɛjŋ';
    // 清青合併前，庚三和庚二是相同的元音
    if (韻圖等 === '三') return 選項.BC韻合流 ? 'j̈əjŋ|ɥ̈əjŋ' : 'j̈ɛjŋ|ɥ̈ɛjŋ';
    if (韻圖等 === '四') return is('青韻') ? 'jejŋ|ɥejŋ' : 'jɛjŋ|ɥɛjŋ';
  }
  if (is('通攝')) {
    if (韻圖等 === '輕') return 'ɵwŋ';
    if (韻圖等 === '一') return 'ɵwŋ';
    if (韻圖等 === '二') return 'ʕɵwŋ';
    if (韻圖等 === '三') return is('東韻') ? 'ɨwŋ' : 'ɥ̈ɵwŋ';
    if (韻圖等 === '四') return is('東韻') ? 'jɨwŋ' : 'ɥɵwŋ';
  }
  if (is('江攝')) {
    if (韻圖等 === '二' && !選項.宕江韻基合流) return 'ʕœwŋ';
    // 銳音歸合口
    if (韻圖等 === '二') return is('幫見影組') ? 'ʕɑŋ' : 'wʕɑŋ';
  }
  if (is('臻攝') && !is('元韻')) {
    if (韻圖等 === '一') return 'ən|un';
    if (韻圖等 === '二') return 'ʕən|ʕun';
    if (韻圖等 === '三') return 選項.BC韻合流 || is('欣文韻') ? 'ɨn|ʉn' : 'j̈in|ɥ̈in';
    if (韻圖等 === '輕') return 'ʉn';
    if (韻圖等 === '四') return 'in|yn';
  }
  if (is('山攝 或 元韻')) {
    if (韻圖等 === '一') return 'ɑn|wɒn';
    if (韻圖等 === '二') return 'ʕan|wʕan';
    if (韻圖等 === '三') return 選項.BC韻合流 || is('元韻') ? 'j̈ɐn|ɥ̈ɐn' : 'j̈ɛn|ɥ̈ɛn';
    if (韻圖等 === '輕') return 'ɐn';
    if (韻圖等 === '四') return 'jɛn|ɥɛn';
  }
  if (is('深攝')) {
    if (韻圖等 === '二') return 'ʕəm';
    if (韻圖等 === '三') return 選項.BC韻合流 ? 'ɨm' : 'j̈im';
    if (韻圖等 === '四') return 'im';
  }
  if (is('咸攝')) {
    if (韻圖等 === '一') return 'ɑm';
    if (韻圖等 === '二') return 'ʕam';
    if (韻圖等 === '三') return 選項.BC韻合流 || is('嚴韻') ? 'j̈ɐm' : 'j̈ɛm';
    if (韻圖等 === '輕') return 'ɐm';
    if (韻圖等 === '四') return 'jɛm';
  }
  throw new Error('無介音規則');
}

function 韻母開合處理() {
  let 韻母列表 = 韻母預處理().split('|');
  if (韻母列表.length === 1) return 韻母列表[0];
  // 一等唇音取合口，二三四等唇音取開口
  if (韻圖等 === '一') return is('開口')? 韻母列表[0] : 韻母列表[1];
  return !is('合口 或 虞韻')? 韻母列表[0] : 韻母列表[1];
}

function get韻母() {
  let 韻母 = 韻母開合處理();
  韻母 = 韻母.replace('ʕ', 選項.二等介音);
  韻母 = 韻母.replace('ɤ', 選項.魚虞模[2]);
  韻母 = 韻母.replace('o', 選項.魚虞模[6]);
  韻母 = 韻母.replace('ɵ', 選項.侯東韻[0]);
  if (選項.幫組一等介音 !== 'w' && is('幫組') && 韻母[0] === 'w') {
    韻母 = 韻母.slice(1);
  }
  if (選項.清青合併 && is('梗攝 三等')) {
    韻母 = 韻母.replace('ɛ', 'e');
  }
  if (is('精莊組 止攝 開口')) {
    韻母 = 韻母.replace('i', 選項.精莊組止攝開口);
  }

  if (is('入聲')) {
    let 陽聲韻尾列表 = ['m', 'n', 'ŋ'];
    let 入聲韻尾列表 = 選項.入聲韻尾.replace(/-+/g, '').split(' ');
    for (let i = 0; i < 陽聲韻尾列表.length; i++) {
      韻母 = 韻母.replace(陽聲韻尾列表[i], 入聲韻尾列表[i]);
    }
  }
  return 韻母;
}

function get聲調() {
  let 聲調 = 選項.全濁上歸去 && is('上聲 全濁') ? '去' : 音韻地位.聲;
  return {
    五度符號: {
      平: is('全清 或 次清') ?
          '˦˨' : '˨˩',
      上: !is('全濁') ?
          '˦˥' : '˨˦',
      去: is('全清 或 次清') ?
          '˧˨˦' : '˨˨˦',
      入: is('全清 或 次清') || (選項.次濁入歸陰入 && is('次濁')) ?
          '˦' : '˨',
    },
    附加符號: { 平: '̀', 上: '́', 去: '̌', 入: '̆' },
    調類數字: { 平: '¹', 上: '²', 去: '³', 入: '⁴' },
  }[選項.聲調][聲調];
}

let 聲母 = get聲母();
let 韻母 = get韻母();
let 聲調 = get聲調();

function get聲調附加符號位置() {
  let 介音列表 = ['̈', 'ʕ', 'j', 'ɥ', 'w'];
  let 介音位置 = -1;
  // 介音後是元音，元音後是插入符號的位置（若有成音節符號則再後移 1）
  let 偏移量 = 韻母.includes('̩') ? 3 : 2;
  for (let i = 0; i < 介音列表.length; i++) {
    let 介音位置 = 韻母.indexOf(介音列表[i]);
    if (介音位置 > -1) {
      // 排除韻尾
      if (介音位置 === 韻母.length - 1) continue;
      if (介音列表[i] === 'w' && 介音位置 === 韻母.length - 2 && is('通攝')) continue;
      return 介音位置 + 偏移量;
    }
  }
  return -1 + 偏移量;
}

if (選項.聲調 === '附加符號') {
  let i = get聲調附加符號位置();
  return 聲母 + 韻母.slice(0, i) + 聲調 + 韻母.slice(i);
}
return 聲母 + 韻母 + 聲調;
